unit MultiLst;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Menus,
  Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
  DsgnIntf;
 
const
 MultiListID = 'TMultiList - Copyright  1996-1997 by Jim Wood';
 ColumnIndent = 6;
 MaxColumns = 15;

type
  TAboutButtonProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

  {Create 10 bitmap placeholders.}
  TBitmapArrayRange = 0..15;
  TListBitmap = (lbBitmap0, lbBitmap1, lbBitmap2, lbBitmap3, lbBitmap4,
                 lbBitmap5, lbBitmap6, lbBitmap7, lbBitmap8, lbBitmap9,
                 lbBitmap10,lbBitmap11,lbBitmap12,lbBitmap13,lbBitmap14,
                 lbBitmap15);
  TListBitmaps = set of TListBitmap;
  TBitmapArray = array[TBitmapArrayRange] of TBitmap;

  TAlign = array[0..MAXCOLUMNS] of TAlignment;

  TMultiList = class(TCustomListBox)
 private
    Header : THeader;
    FIndexProperty: Integer;
    FIndexDefault: Boolean;
    FAbout: TAboutButtonProperty;
    FUserBitmaps: TListBitmaps;
    FOldBitmaps: TListBitmaps;
    FPictures: TBitmapArray;
    FFieldSizing: Boolean;
    FShowColLines: Boolean;
    FShowParentHighlight: Boolean;
    FHeaderHeight: Integer;
    FHeaderFont: TFont;
    FColLineColor: TColor;
    FHighlightColor: TColor;
    FChildHighlightTextColor: TColor;
    FHighlightText: TColor;
    FMaskColor: TColor;
    FSeperator: string;
    FHeaderSections: TStringList;
    FScrollSpeed: Integer;
    FSectionWidth0: Integer;
    FSectionWidth1: Integer;
    FSectionWidth2: Integer;
    FSectionWidth3: Integer;
    FSectionWidth4: Integer;
    FSectionWidth5: Integer;
    FSectionWidth6: Integer;
    FSectionWidth7: Integer;
    FSectionWidth8: Integer;
    FSectionWidth9: Integer;
    FSectionWidth10: Integer;
    FSectionWidth11: Integer;
    FSectionWidth12: Integer;
    FSectionWidth13: Integer;
    FSectionWidth14: Integer;
    FUpdateAlways: Boolean;
    FHScroll: Boolean;
    FScrollWidth: Integer;
    FAlignment: TAlign;
    FirstSize: Boolean;
    ScrollClick: Boolean;
    FIgnoreBitmap: Boolean;
    OldSectionCount: Integer;
    OldHeaderWidth: Integer;
    OldListWidth: Integer;
    OldTopIndex: Integer;
    FOnMeasureItem: TMeasureItemEvent;
    procedure UpdateList(ListTop: Integer; ListLeft: Integer; ListBottom: Integer;
                         ListRight: Integer; ScrollNumber: Integer);
    procedure ListHeaderSized(Sender: TObject; ASection, AWidth: Integer);
    procedure SetScrollSpeed(Value: Integer);
    procedure SetSectionWidth0(Value: Integer);
    procedure SetSectionWidth1(Value: Integer);
    procedure SetSectionWidth2(Value: Integer);
    procedure SetSectionWidth3(Value: Integer);
    procedure SetSectionWidth4(Value: Integer);
    procedure SetSectionWidth5(Value: Integer);
    procedure SetSectionWidth6(Value: Integer);
    procedure SetSectionWidth7(Value: Integer);
    procedure SetSectionWidth8(Value: Integer);
    procedure SetSectionWidth9(Value: Integer);
    procedure SetSectionWidth10(Value: Integer);
    procedure SetSectionWidth11(Value: Integer);
    procedure SetSectionWidth12(Value: Integer);
    procedure SetSectionWidth13(Value: Integer);
    procedure SetSectionWidth14(Value: Integer);
    procedure SetHeaderHeight(Value: Integer);
    procedure SetShowParentHighlight(Value: Boolean);
    procedure SetShowLines(Value: Boolean);
    procedure SetIndexDefault(Value: Boolean);
    procedure SetHeaderFont(Value: TFont);
    procedure SetColLineColor(Value: TColor);
    procedure SetHighlightColor(Value: TColor);
    procedure SetHighlightText(Value: TColor);
    procedure SetChildHighlightTextColor(Value: TColor);
    procedure SetMaskColor(Value: TColor);
    procedure SetColumnSeperator(Value: string);
    procedure SetHeaderSections(Value: TStringList);
    procedure ClearBitmap(var Bitmap: TBitmap; Kind: TListBitmap);
    procedure ChangeBitmap(Value: TBitmap; Kind: TListBitmap);
    procedure SetPicture(Index: Integer; Value: TBitmap);
    procedure SetHScroll(Value: Boolean);
    procedure SetScrollWidth(Value: Integer);
    function  GetAlignment(Index: Integer): TAlignment;
    procedure SetAlignment(Index: Integer; Value: TAlignment);
    function  GetPicture(Index: Integer): TBitmap;
    function  StoreBitmap(Index: Integer): Boolean;
    procedure ReadBinaryData(Stream: TStream);
    procedure WriteBinaryData(Stream: TStream);
    procedure CreateGlyph;
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
  {  procedure WMScrollClick(var Message: TWMVScroll); message WM_VSCROLL; }
  protected
    procedure DrawItem(Index: Integer; Rect: TRect;
      State: TOwnerDrawState); override;
    procedure DefineProperties(Filer: TFiler); override;
  public
    constructor Create(AOwner:TComponent); override;
    destructor  Destroy; override;
    function    ItemBitmap(ListIndex: Integer): Integer;
    function    ItemParent(ListIndex: Integer): string;
    function    ItemChild(ColIndex: Integer; ListIndex: Integer): string;
    procedure   SetItemBitmap(ListIndex: Integer; BitmapIndex: Integer);
    procedure   SetItemParent(ListIndex: Integer; ChangeItem: string);
    procedure   SetItemChild(ColIndex: Integer; ListIndex: Integer; ChangeItem: string);
  published
    property About: TAboutButtonProperty read FAbout write FAbout;
    property Alignment0: TAlignment index 0 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment1: TAlignment index 1 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment2: TAlignment index 2 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment3: TAlignment index 3 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment4: TAlignment index 4 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment5: TAlignment index 5 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment6: TAlignment index 6 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment7: TAlignment index 7 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment8: TAlignment index 8 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment9: TAlignment index 9 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment_10: TAlignment index 10 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment_11: TAlignment index 11 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment_12: TAlignment index 12 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment_13: TAlignment index 13 read GetAlignment write SetAlignment default taLeftJustify;
    property Alignment_14: TAlignment index 14 read GetAlignment write SetAlignment default taLeftJustify;
    property SectionWidth0 : Integer read FSectionWidth0 write SetSectionWidth0;
    property SectionWidth1 : Integer read FSectionWidth1 write SetSectionWidth1;
    property SectionWidth2 : Integer read FSectionWidth2 write SetSectionWidth2;
    property SectionWidth3 : Integer read FSectionWidth3 write SetSectionWidth3;
    property SectionWidth4 : Integer read FSectionWidth4 write SetSectionWidth4;
    property SectionWidth5 : Integer read FSectionWidth5 write SetSectionWidth5;
    property SectionWidth6 : Integer read FSectionWidth6 write SetSectionWidth6;
    property SectionWidth7 : Integer read FSectionWidth7 write SetSectionWidth7;
    property SectionWidth8 : Integer read FSectionWidth8 write SetSectionWidth8;
    property SectionWidth9 : Integer read FSectionWidth9 write SetSectionWidth9;
    property SectionWidth_10 : Integer read FSectionWidth10 write SetSectionWidth10;
    property SectionWidth_11 : Integer read FSectionWidth11 write SetSectionWidth11;
    property SectionWidth_12 : Integer read FSectionWidth12 write SetSectionWidth12;
    property SectionWidth_13 : Integer read FSectionWidth13 write SetSectionWidth13;
    property SectionWidth_14 : Integer read FSectionWidth14 write SetSectionWidth14;
    property SetHorizontalScroll: Boolean read FHScroll write SetHScroll;
    property DefaultItemSelected: Boolean read FIndexDefault write SetIndexDefault;
    property ScrollWidth: Integer read FScrollWidth write SetScrollWidth;
    property HeaderHeight : Integer read FHeaderHeight write SetHeaderHeight;
    property HeaderFont : TFont read FHeaderFont write SetHeaderFont;
    property ChildHighlightTextColor : TColor read FChildHighlightTextColor write SetChildHighlightTextColor;
    property ColumnLineColor : TColor read FColLineColor write SetColLineColor;
    property HighlightColor : TColor read FHighlightColor write SetHighlightColor;
    property HighlightText : TColor read FHighlightText write SetHighlightText;
    property BitmapMaskColor : TColor read FMaskColor write SetMaskColor;
    property ListBitmap0 : TBitmap index 0 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap1 : TBitmap index 1 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap2 : TBitmap index 2 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap3 : TBitmap index 3 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap4 : TBitmap index 4 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap5 : TBitmap index 5 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap6 : TBitmap index 6 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap7 : TBitmap index 7 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap8 : TBitmap index 8 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap9 : TBitmap index 9 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap_10 : TBitmap index 10 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap_11 : TBitmap index 11 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap_12 : TBitmap index 12 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap_13 : TBitmap index 13 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap_14 : TBitmap index 14 read GetPicture write SetPicture stored StoreBitmap;
    property ListBitmap_15 : TBitmap index 15 read GetPicture write SetPicture stored StoreBitmap;
    property SmoothScroll : Boolean read FFieldSizing write FFieldSizing;
    property ScrollSpeed : Integer read FScrollSpeed write SetScrollSpeed;
    property ShowColumnLines : Boolean read FShowColLines write SetShowLines;
    property ShowParentHighlight : Boolean read FShowParentHighlight write SetShowParentHighlight;
    property HeaderSections : TStringList read FHeaderSections write SetHeaderSections;
    property ColumnSeperator : string read FSeperator write SetColumnSeperator;
    property IgnoreBitmap: Boolean read FIgnoreBitmap write FIgnoreBitmap;
    property UpdateAlways: Boolean read FUpdateAlways write FUpdateAlways;
    property Align;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property Cursor;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ExtendedSelect;
    property Font;
    property Height;
    property HelpContext;
    property Hint;
    property ItemHeight;
    property Items;
    property Left;
    property MultiSelect;
    property Name;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Tag;
    property Top;
    property Visible;
    property Width;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
end;

procedure Register;

implementation

procedure TAboutButtonProperty.Edit;
var
   AboutTitle,
   AboutDesc: String;
begin
{$IFDEF WIN32}
   AboutTitle := 'About TMultiList (32-bit)';
{$ELSE}
   AboutTitle := 'About TMultiList (16-bit)';
{$ENDIF}
   AboutDesc := 'TMultiList (Version 3.0)'+^M+'[Registered Version]'+^M^M+'Copyright  1996 by Jim Wood';
{$IFDEF NOREG}
   AboutDesc := 'TMultiList (Version 3.0)'+^M+'[UnRegistered Shareware Version]'+^M^M+'Copyright  1996 by Jim Wood';
{$ENDIF}
   Application.MessageBox(PChar(AboutDesc), PChar(AboutTitle), MB_OK+ MB_ICONINFORMATION);
end;

function TAboutButtonProperty.GetAttributes: TPropertyAttributes;
begin
 	Result := [paMultiSelect, paDialog, paReadOnly];
end;

function TAboutButtonProperty.GetValue: string;
begin
	Result := '(About)';
end;

procedure Register;
begin
  {Register the Component on the Additional page}
  RegisterComponents('Additional',[TMultiList]);
  RegisterPropertyEditor(TypeInfo(TAboutButtonProperty), TMultiList, 'ABOUT', TAboutButtonProperty);
end;
 
const
  Images: array[TBitmapArrayRange] of PChar = ('BITMAP1', 'BITMAP2', 'BITMAP3', 'BITMAP4', 'BITMAP5',
                                               'BITMAP6', 'BITMAP7', 'BITMAP8', 'BITMAP9', 'BITMAP10',
                                               'BITMAP11','BITMAP12','BITMAP13','BITMAP14','BITMAP15',
                                               'BITMAP16');

(*******************************************************
* constructor TMultiList.Create();
*
* Author: Jim Wood
*
* Purpose: This constructor is called each time the
*          component is dropped onto a form for the
*          first time.
*
* Caveats: None
*
*******************************************************)
constructor TMultiList.Create(AOwner:TComponent);
var
 I,
 ListWidth: Integer;
begin
  inherited Create(AOwner);
    {Set up component defaults}
    CreateGlyph;
    OldListWidth := 0;
    OldSectionCount := 0;
    FIndexDefault := TRUE;
    FUpdateAlways := TRUE;
    FIndexProperty := 1;
    FScrollSpeed := 5;
    Style := lbOwnerDrawVariable;
    FShowColLines := FALSE;
    FShowParentHighlight := FALSE;
    {Default seoerator is a PIPE character.}
    FSeperator := '|';
    FHeaderSections := TStringList.Create;
    FHeaderFont := TFont.Create;
    FMaskColor := clPurple;
    FColLineColor := clGray;
    FChildHighlightTextColor := clHighlight;
    FHighlightColor := clHighlight;
    FHighlightText := clHighlightText;
    FirstSize := TRUE;
    {Item height and HeaderHeight should be the same value for best performance.}
    ItemHeight := 18;
    FHeaderHeight := 18;
    FHeaderFont.Name := 'MS Sans Serif';
    FHeaderFont.Size := 8;
    FHeaderFont.Pitch := fpDefault;
    FHeaderFont.Height := -12;
    FHeaderFont.Color := clBlack;
    FHeaderFont.Style := [];
    ListWidth := Width;
    FScrollWidth := Width;
   begin
     if FindComponent('ListHeader') = nil then
      begin { if the component 'ListHeader' does not already exist. }
       with THeader.Create(Self) do
        begin
         Parent := Self;
         Top := 0;
         Height := FHeaderHeight;
         Width := ListWidth;
         Name := 'ListHeader';             { Set ListHeader's component name. }
         Font.Assign(FHeaderFont);
         Visible := true;
        end;
      end;
      for I := 0 to ComponentCount -1 do
      if Components[I] is THeader then
        if THeader(Components[I]).Name = 'ListHeader' then Header := THeader(Components[I]);
     end;
     Font.Name := 'MS Sans Serif';
     Font.Size := 8;
     FHeaderSections.Clear;
end;

(*******************************************************
* procedure TMultiList.UpdateList();
*
* Author: Jim Wood
*
* Purpose: This procedure is called each time an update
*          needs to be made to the MultiList ListBox.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.UpdateList(ListTop: Integer; ListLeft: Integer; ListBottom: Integer;
                                ListRight: Integer; ScrollNumber: Integer);
var
 FillRegion: TRect;
 ScrollPosition: Integer;
begin
 ScrollPosition := GetScrollPos(Handle, SB_HORZ);
 Canvas.Brush.Color := Color;
 FillRegion.Top := ListTop;
 FillRegion.Left := (ListLeft+1)-ScrollPosition;
 FillRegion.Right := ListRight;
 FillRegion.Bottom := ListBottom;
 if (FShowColLines = TRUE) and (ScrollNumber > 0) then
  begin
   Canvas.Pen.Color := Color;
   Canvas.MoveTo(ListLeft,Header.Height);
   Canvas.LineTo(ListLeft,Height);
  end
 else
  begin
   FillRegion.Left := FillRegion.Left-1;
  end;
 if ScrollNumber < 0 then
  begin
   while ScrollNumber < 0 do
    begin
     if ScrollNumber+FScrollSpeed <= 0 then
      begin
       ScrollNumber := ScrollNumber + FScrollSpeed;
       ScrollWindowEx(Handle, 0-FScrollSpeed, 0, nil, @FillRegion, 0, nil, SW_INVALIDATE);
      end
     else
      begin
       ScrollNumber := ScrollNumber + 1;
       ScrollWindowEx(Handle, -1, 0, nil, @FillRegion, 0, nil, SW_INVALIDATE);
      end;
    end;
  end
 else
  begin
   while ScrollNumber > 0 do
    begin
     if ScrollNumber-FScrollSpeed >= 0 then
      begin
       ScrollNumber := ScrollNumber - FScrollSpeed;
       ScrollWindowEx(Handle, 0+FScrollSpeed, 0, nil, @FillRegion, 0, nil, SW_INVALIDATE);
      end
     else
      begin
       ScrollNumber := ScrollNumber - 1;
       ScrollWindowEx(Handle, 1, 0, nil, @FillRegion, 0, nil, SW_INVALIDATE);
      end;
    end;
  end;
 if (FirstSize = TRUE) then
  begin
   FirstSize := FALSE;
   Invalidate;
  end
 else
 if (FShowColLines = TRUE) then
  begin
   FillRegion.Left := FillRegion.Left - 1;
   InvalidateRect(Handle, @FillRegion, TRUE);
  end;
end;

(*******************************************************
* procedure TMultiList.ListHeaderSized();
*
* Author: Jim Wood
*
* Purpose: This procedure is called each time the List
*          header is resized.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.ListHeaderSized(Sender: TObject; ASection, AWidth: Integer);
var
 Section,
 AddLeft: Integer;
 OldSection: array[0..MaxColumns] of Integer;
begin
 {If the width of the header changes size, then...}
 if FUpdateAlways then
    Invalidate;
 if AWidth <> OldHeaderWidth then
  begin
   {Save old section widths to determine # of pixels to shift list column(s)}
   OldHeaderWidth := AWidth;
   OldSection[0] := FSectionWidth0;
   OldSection[1] := FSectionWidth1;
   OldSection[2] := FSectionWidth2;
   OldSection[3] := FSectionWidth3;
   OldSection[4] := FSectionWidth4;
   OldSection[5] := FSectionWidth5;
   OldSection[6] := FSectionWidth6;
   OldSection[7] := FSectionWidth7;
   OldSection[8] := FSectionWidth8;
   OldSection[9] := FSectionWidth9;
   OldSection[10] := FSectionWidth10;
   OldSection[11] := FSectionWidth11;
   OldSection[12] := FSectionWidth12;
   OldSection[13] := FSectionWidth13;
   OldSection[14] := FSectionWidth14;

   {Set local property values := current THeader section widths}
   FSectionWidth0 := Header.SectionWidth[0];
   FSectionWidth1 := Header.SectionWidth[1];
   FSectionWidth2 := Header.SectionWidth[2];
   FSectionWidth3 := Header.SectionWidth[3];
   FSectionWidth4 := Header.SectionWidth[4];
   FSectionWidth5 := Header.SectionWidth[5];
   FSectionWidth6 := Header.SectionWidth[6];
   FSectionWidth7 := Header.SectionWidth[7];
   FSectionWidth8 := Header.SectionWidth[8];
   FSectionWidth9 := Header.SectionWidth[9];
   FSectionWidth10 := Header.SectionWidth[10];
   FSectionWidth11 := Header.SectionWidth[11];
   FSectionWidth12 := Header.SectionWidth[12];
   FSectionWidth13 := Header.SectionWidth[13];
   FSectionWidth14 := Header.SectionWidth[14];
   AddLeft := 0;
   for Section := 0 to ASection do
    begin
     if OldSection[Section] < Header.SectionWidth[Section] then
      begin
       AddLeft := AddLeft + OldSection[Section];
      end
     else
      begin
       AddLeft := AddLeft + Header.SectionWidth[Section];
      end;
    end;
   AddLeft := AddLeft - AWidth;
   if OldSection[0] > 0 then
    UpdateList(Header.Height,AddLeft+Header.SectionWidth[ASection],
               Height,Width,Header.SectionWidth[ASection]-OldSection[ASection])
   else
    begin
     Invalidate;
    end;
  end
end;

(*******************************************************
* procedure TMultiList.CNDrawItem();
*
* Author: Jim Wood
*
* Purpose: This procedure is called each time the List
*          needs to be drawn.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.CNDrawItem(var Message: TWMDrawItem);
var
  ListRect: TRect;
  State: TOwnerDrawState;
  I,
  x,
  NewTopIndex: Integer;
begin
 if not (csDesigning in ComponentState) then
 begin
  { Set the horizontal scroll bar if FHScroll = TRUE  }
  { The entire functionality is handled by sending    }
  { a resize message to the listbox bigger than its   }
  { current instance size; this triggers the creation }
  { of an internal horizontal scrollbar.              }
  if FHScroll then
   begin
    SendMessage( Handle ,
                 LB_SetHorizontalExtent ,
                 FScrollWidth ,
                 LongInt( 0 ) );
   end
  else
   begin
    SendMessage( Handle ,
                 LB_SetHorizontalExtent ,
                 0 ,
                 LongInt( 0 ) );
   end;
  for I := 0 to ComponentCount -1 do
   if Components[I] is THeader then
    if THeader(Components[I]).Name = 'ListHeader' then
     begin
      Header := THeader(Components[I]);
      if Header.Sections.Count <> FHeaderSections.Count then
       Header.Sections.Assign(FHeaderSections);
      if FFieldSizing then
       Header.OnSizing := ListHeaderSized
      else
       Header.OnSized := ListHeaderSized;
       if (OldListWidth <> Width) then
        begin
         if (Header.Sections.Count > 0) and (FSectionWidth0 > 0) then Header.SectionWidth[0] := FSectionWidth0;
         if (Header.Sections.Count > 1) and (FSectionWidth1 > 0) then Header.SectionWidth[1] := FSectionWidth1;
         if (Header.Sections.Count > 2) and (FSectionWidth2 > 0) then Header.SectionWidth[2] := FSectionWidth2;
         if (Header.Sections.Count > 3) and (FSectionWidth3 > 0) then Header.SectionWidth[3] := FSectionWidth3;
         if (Header.Sections.Count > 4) and (FSectionWidth4 > 0) then Header.SectionWidth[4] := FSectionWidth4;
         if (Header.Sections.Count > 5) and (FSectionWidth5 > 0) then Header.SectionWidth[5] := FSectionWidth5;
         if (Header.Sections.Count > 6) and (FSectionWidth6 > 0) then Header.SectionWidth[6] := FSectionWidth6;
         if (Header.Sections.Count > 7) and (FSectionWidth7 > 0) then Header.SectionWidth[7] := FSectionWidth7;
         if (Header.Sections.Count > 8) and (FSectionWidth8 > 0) then Header.SectionWidth[8] := FSectionWidth8;
         if (Header.Sections.Count > 9) and (FSectionWidth9 > 0) then Header.SectionWidth[9] := FSectionWidth9;
         if (Header.Sections.Count > 10) and (FSectionWidth10 > 0) then Header.SectionWidth[10] := FSectionWidth10;
         if (Header.Sections.Count > 11) and (FSectionWidth11 > 0) then Header.SectionWidth[11] := FSectionWidth11;
         if (Header.Sections.Count > 12) and (FSectionWidth12 > 0) then Header.SectionWidth[12] := FSectionWidth12;
         if (Header.Sections.Count > 13) and (FSectionWidth13 > 0) then Header.SectionWidth[13] := FSectionWidth13;
         if (Header.Sections.Count > 14) and (FSectionWidth14 > 0) then Header.SectionWidth[14] := FSectionWidth14;
        end;
{       for x := 1 to Header.Sections.Count-1 do
        begin
         if Header.SectionWidth[x] = 0 then
          Header.SectionWidth[x-1] := Header.SectionWidth[x-1]+(FinalWidth - Width);
        end;  }
     end;
  end
  else
  if (OldListWidth <> Width) then
   begin
      if FFieldSizing then
       Header.OnSizing := ListHeaderSized
      else
       Header.OnSized := ListHeaderSized;
     if (Header.Sections.Count > 0) and (FSectionWidth0 > 0) and (Header.SectionWidth[0] <> FSectionWidth0) then
      begin
       Header.SectionWidth[0] := FSectionWidth0;
      end;
     if (Header.Sections.Count > 1) and (FSectionWidth1 > 0) and (Header.SectionWidth[1] <> FSectionWidth1) then
      begin
       Header.SectionWidth[1] := FSectionWidth1;
      end;
     if (Header.Sections.Count > 2) and (FSectionWidth2 > 0) and (Header.SectionWidth[2] <> FSectionWidth2) then
      begin
       Header.SectionWidth[2] := FSectionWidth2;
      end;
     if (Header.Sections.Count > 3) and (FSectionWidth3 > 0) and (Header.SectionWidth[3] <> FSectionWidth3) then
      begin
       Header.SectionWidth[3] := FSectionWidth3;
      end;
     if (Header.Sections.Count > 4) and (FSectionWidth4 > 0) and (Header.SectionWidth[4] <> FSectionWidth4) then
      begin
       Header.SectionWidth[4] := FSectionWidth4;
      end;
     if (Header.Sections.Count > 5) and (FSectionWidth5 > 0) and (Header.SectionWidth[5] <> FSectionWidth5) then
      begin
       Header.SectionWidth[5] := FSectionWidth5;
      end;
     if (Header.Sections.Count > 6) and (FSectionWidth6 > 0) and (Header.SectionWidth[6] <> FSectionWidth6) then
      begin
       Header.SectionWidth[6] := FSectionWidth6;
      end;
     if (Header.Sections.Count > 7) and (FSectionWidth7 > 0) and (Header.SectionWidth[7] <> FSectionWidth7) then
      begin
       Header.SectionWidth[7] := FSectionWidth7;
      end;
     if (Header.Sections.Count > 8) and (FSectionWidth8 > 0) and (Header.SectionWidth[8] <> FSectionWidth8) then
      begin
       Header.SectionWidth[8] := FSectionWidth8;
      end;
     if (Header.Sections.Count > 9) and (FSectionWidth9 > 0) and (Header.SectionWidth[9] <> FSectionWidth9) then
      begin
       Header.SectionWidth[9] := FSectionWidth9;
      end;
     if (Header.Sections.Count > 10) and (FSectionWidth0 > 0) and (Header.SectionWidth[10] <> FSectionWidth10) then
      begin
       Header.SectionWidth[10] := FSectionWidth10;
      end;
     if (Header.Sections.Count > 11) and (FSectionWidth1 > 0) and (Header.SectionWidth[11] <> FSectionWidth11) then
      begin
       Header.SectionWidth[11] := FSectionWidth11;
      end;
     if (Header.Sections.Count > 12) and (FSectionWidth2 > 0) and (Header.SectionWidth[12] <> FSectionWidth12) then
      begin
       Header.SectionWidth[12] := FSectionWidth12;
      end;
     if (Header.Sections.Count > 13) and (FSectionWidth3 > 0) and (Header.SectionWidth[13] <> FSectionWidth13) then
      begin
       Header.SectionWidth[13] := FSectionWidth13;
      end;
     if (Header.Sections.Count > 14) and (FSectionWidth4 > 0) and (Header.SectionWidth[14] <> FSectionWidth14) then
      begin
       Header.SectionWidth[14] := FSectionWidth14;
      end;
   end
  else
   begin
    if (Header.Sections.Count > 0) and (FSectionWidth0 > 0) then Header.SectionWidth[0] := FSectionWidth0;
    if (Header.Sections.Count > 1) and (FSectionWidth1 > 0) then Header.SectionWidth[1] := FSectionWidth1;
    if (Header.Sections.Count > 2) and (FSectionWidth2 > 0) then Header.SectionWidth[2] := FSectionWidth2;
    if (Header.Sections.Count > 3) and (FSectionWidth3 > 0) then Header.SectionWidth[3] := FSectionWidth3;
    if (Header.Sections.Count > 4) and (FSectionWidth4 > 0) then Header.SectionWidth[4] := FSectionWidth4;
    if (Header.Sections.Count > 5) and (FSectionWidth5 > 0) then Header.SectionWidth[5] := FSectionWidth5;
    if (Header.Sections.Count > 6) and (FSectionWidth6 > 0) then Header.SectionWidth[6] := FSectionWidth6;
    if (Header.Sections.Count > 7) and (FSectionWidth7 > 0) then Header.SectionWidth[7] := FSectionWidth7;
    if (Header.Sections.Count > 8) and (FSectionWidth8 > 0) then Header.SectionWidth[8] := FSectionWidth8;
    if (Header.Sections.Count > 9) and (FSectionWidth9 > 0) then Header.SectionWidth[9] := FSectionWidth9;
    if (Header.Sections.Count > 10) and (FSectionWidth10 > 0) then Header.SectionWidth[10] := FSectionWidth10;
    if (Header.Sections.Count > 11) and (FSectionWidth11 > 0) then Header.SectionWidth[11] := FSectionWidth11;
    if (Header.Sections.Count > 12) and (FSectionWidth12 > 0) then Header.SectionWidth[12] := FSectionWidth12;
    if (Header.Sections.Count > 13) and (FSectionWidth13 > 0) then Header.SectionWidth[13] := FSectionWidth13;
    if (Header.Sections.Count > 14) and (FSectionWidth14 > 0) then Header.SectionWidth[14] := FSectionWidth14;
    if OldSectionCount = 0 then Invalidate;
    OldSectionCount := Header.Sections.Count;
   end;
  OldListWidth := Width;
  Header.Top := 0;
  Header.Height := FHeaderHeight;
  Header.Font.Assign(FHeaderFont);
  if Items.Count > 0 then
   begin
   if (Items[0] <> MultiListID) then
    begin
     Items.Add('');
     for x := Items.Count-2 downto 0 do
      begin
       Items[x+1] := Items[x];
      end;
     Items[0] := MultiListID;
     ItemIndex := FIndexProperty;
    end;
   end;
  with Message.DrawItemStruct^ do
  begin
    ListRect := rcItem;
    if (FHScroll = TRUE) and (FScrollWidth > Width) then ListRect.Right := FScrollWidth;
    if (FHScroll = TRUE) and (Header.Width < FScrollWidth) then
     begin
      Header.Width := FScrollWidth;
      FSectionWidth0 := Header.SectionWidth[0];
      FSectionWidth1 := Header.SectionWidth[1];
      FSectionWidth2 := Header.SectionWidth[2];
      FSectionWidth3 := Header.SectionWidth[3];
      FSectionWidth4 := Header.SectionWidth[4];
      FSectionWidth5 := Header.SectionWidth[5];
      FSectionWidth6 := Header.SectionWidth[6];
      FSectionWidth7 := Header.SectionWidth[7];
      FSectionWidth8 := Header.SectionWidth[8];
      FSectionWidth9 := Header.SectionWidth[9];
      FSectionWidth10 := Header.SectionWidth[10];
      FSectionWidth11 := Header.SectionWidth[11];
      FSectionWidth12 := Header.SectionWidth[12];
      FSectionWidth13 := Header.SectionWidth[13];
      FSectionWidth14 := Header.SectionWidth[14];
     end;
    NewTopIndex := Perform(LB_GETTOPINDEX,0,0);
    if OldTopIndex <> NewTopIndex then Perform(LB_SETITEMHEIGHT, OldTopIndex, Longint(ItemHeight));
    Perform(LB_SETITEMHEIGHT, NewTopIndex, Longint(Header.Height));
    if (OldTopIndex < NewTopIndex) and (ItemHeight <> Header.Height) then Invalidate;
    OldTopIndex := NewTopIndex;
    if ListRect.Top >= Header.Height then
    begin
     State := TOwnerDrawState(WordRec(LongRec(itemState).Lo).Lo);
     Canvas.Handle := hDC;
     Canvas.Font := Font;
     Canvas.Brush := Brush;
     if (Integer(itemID) >= 0) and (odSelected in State) then
     begin
       Canvas.Brush.Color := FHighlightColor;
       Canvas.Font.Color := FHighlightText
     end;
     if Integer(itemID) >= 0 then
       DrawItem(itemID, ListRect, State) else
       Canvas.FillRect(ListRect);
     if odFocused in State then
      begin
       if (FShowParentHighlight = TRUE) then ListRect.Right := Header.SectionWidth[0]+1;
       DrawFocusRect(hDC, ListRect);
      end;
     Canvas.Handle := 0;
    end;
  end;
end;

(*******************************************************
* procedure TMultiList.DrawItem();
*
* Author: Jim Wood
*
* Purpose: This procedure is called each time an individual
*          item needs to be drawn.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.DrawItem(Index: Integer; Rect: TRect;
      State: TOwnerDrawState);
var
 FillRegion,
 DrawTextRect: TRect;
 s: string;
 DrawColumns,
 i: integer;
 Bitmap: TBitmap;
 UseHeader,
 DrawBitmap: Boolean;
 UFormat: UINT;
 {$IFDEF WIN32}
 DrawTextParams: TDrawTextParams;
 {$ENDIF}

  function GetBitmap(Value: TListBitmap): TBitmap;
  begin
    Result := FPictures[Ord(Value)];
  end;

  function FindSectionWidth(Section: Integer): Integer;
  var
   AddSections: Integer;
  begin
   Result := 0;
   for AddSections := 0 to Section do
    begin
     Result := Result + Header.SectionWidth[AddSections];
    end;
  end;

begin
      s:= Items[Index];
      UseHeader := false;
      try
       if Header <> nil then
        begin
         UseHeader := true;
         Header.Sections.Assign(FHeaderSections);
        end;
      except
      end;
      if UseHeader then
       begin
        if (FHScroll = TRUE) and (FScrollWidth > Width) then
         Header.Width := FScrollWidth
        else
         Header.Width := Width;
        with Canvas do
        begin
                if Rect.Top >= Header.Height then
                 begin
                  FillRegion.Top := Rect.Top;
                  FillRegion.Left := Rect.Left;
                  FillRegion.Bottom := Rect.Bottom;
                  FillRegion.Right := Rect.Left+Header.SectionWidth[0];
                  FillRect(FillRegion);
                 end;
                DrawBitmap := false;
                Bitmap:=nil;
                case s[1] of
                 '0': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap0); end;
                 '1': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap1); end;
                 '2': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap2); end;
                 '3': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap3); end;
                 '4': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap4); end;
                 '5': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap5); end;
                 '6': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap6); end;
                 '7': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap7); end;
                 '8': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap8); end;
                 '9': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap9); end;
                 '*': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap10); end;
                 '!': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap11); end;
                 '@': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap12); end;
                 '#': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap13); end;
                 '$': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap14); end;
                 '%': begin DrawBitmap := true; Bitmap:=GetBitmap(lbBitmap15); end;
                end;
                i:=Pos(FSeperator,s);
                {$IFDEF WIN32}
                DrawTextParams.iTabLength := 3; //Default TabSize
                DrawTextParams.iLeftMargin := 0;
                DrawTextParams.iRightMargin := 0;
                DrawTextParams.cbSize := SizeOf(DrawTextParams);
                {$ENDIF}
                if DrawBitmap then
                 begin
                  BrushCopy(Bounds(Rect.Left+ColumnIndent, Rect.Top, Bitmap.Width, Bitmap.Height), Bitmap,
                            Bounds(0, 0, Bitmap.Width, Bitmap.Height), FMaskColor);	{ render the bitmap }
                  DrawTextRect.Top := Rect.Top+1;
                  DrawTextRect.Left := Rect.Left+Bitmap.Width+10;
                  DrawTextRect.Right := FindSectionWidth(0);
                  DrawTextRect.Bottom := Rect.Bottom;
                  {$IFDEF WIN32}
                  UFormat := DT_EXPANDTABS or DT_END_ELLIPSIS;
                  {$ELSE}
                  UFormat := DT_EXPANDTABS;
                  {$ENDIF}
                  case FAlignment[0] of
                     taLeftJustify  : UFormat := uFormat or DT_LEFT;
                     taRightJustify : UFormat := uFormat or DT_RIGHT;
                     taCenter       : UFormat := uFormat or DT_CENTER;
                  end;
                  {$IFDEF WIN32}
                  DrawTextEx(Handle, PChar(Copy(s,2,i-2)), Length(Copy(s,2,i-2)), DrawTextRect, UFormat, @DrawTextParams);
                  {$ELSE}
                  DrawText(Handle, PChar(Copy(s,2,i-2)), Length(Copy(s,2,i-2)), DrawTextRect, UFormat);
                  {$ENDIF}

                 end
                else
                 begin
                  DrawTextRect.Top := Rect.Top+1;
                  DrawTextRect.Left := Rect.Left+1;
                  DrawTextRect.Right := FindSectionWidth(0);
                  DrawTextRect.Bottom := Rect.Bottom;

                  {$IFDEF WIN32}
                  UFormat := DT_EXPANDTABS or DT_END_ELLIPSIS;
                  {$ELSE}
                  UFormat := DT_EXPANDTABS;
                  {$ENDIF}
                  case FAlignment[0] of
                     taLeftJustify  : UFormat := uFormat or DT_LEFT;
                     taRightJustify : UFormat := uFormat or DT_RIGHT;
                     taCenter       : UFormat := uFormat or DT_CENTER;
                  end;
                  if s[1] = '~' then
                   {$IFDEF WIN32}
                   DrawTextEx(Handle, PChar(Copy(s,2,i-2)), Length(Copy(s,2,i-2)), DrawTextRect, UFormat, @DrawTextParams)
                   {$ELSE}
                   DrawText(Handle, PChar(Copy(s,2,i-2)), Length(Copy(s,2,i-2)), DrawTextRect, UFormat)
                   {$ENDIF}
                  else
                   {$IFDEF WIN32}
                   DrawTextEx(Handle, PChar(Copy(s,1,i-1)), Length(Copy(s,1,i-1)), DrawTextRect, UFormat, @DrawTextParams);
                   {$ELSE}
                   DrawText(Handle, PChar(Copy(s,1,i-1)), Length(Copy(s,1,i-1)), DrawTextRect, UFormat);
                   {$ENDIF}
                 end;
                if (odSelected in State) and (FShowParentHighlight = TRUE) then
                 begin
                  Canvas.Brush.Color := Color;
                  Canvas.Font.Color := FChildHighlightTextColor;
                 end;
                Delete(s,1,i);
                i:=Pos(FSeperator,s);
                FillRegion.Top := Rect.Top;
                FillRegion.Bottom := Rect.Bottom;
                for DrawColumns := 0 to MaxColumns do
                 begin
                  FillRegion.Left := FindSectionWidth(DrawColumns);
                  FillRegion.Right := FindSectionWidth(DrawColumns+1);
                  Canvas.FillRect(FillRegion);

                  DrawTextRect.Top := Rect.Top+1;
                  DrawTextRect.Left := FindSectionWidth(DrawColumns)+ColumnIndent;
                  DrawTextRect.Right := FindSectionWidth(DrawColumns+1);
                  DrawTextRect.Bottom := Rect.Bottom;

                  {$IFDEF WIN32}
                  UFormat := DT_EXPANDTABS or DT_END_ELLIPSIS;
                  {$ELSE}
                  UFormat := DT_EXPANDTABS;
                  {$ENDIF}
                  case FAlignment[DrawColumns+1] of
                     taLeftJustify  : UFormat := uFormat or DT_LEFT;
                     taRightJustify : UFormat := uFormat or DT_RIGHT;
                     taCenter       : UFormat := uFormat or DT_CENTER;
                  end;
                  {$IFDEF WIN32}
                  DrawTextEx(Handle, PChar(Copy(s,1,i-1)), Length(Copy(s,1,i-1)), DrawTextRect, UFormat, @DrawTextParams);
                  {$ELSE}
                  DrawText(Handle, PChar(Copy(s,1,i-1)), Length(Copy(s,1,i-1)), DrawTextRect, UFormat);
                  {$ENDIF}
                  Delete(s,1,i);
                  i:=Pos(FSeperator,s);
                 end;
                if FShowColLines = TRUE then
                 begin
                  Canvas.Pen.Color := FColLineColor;
                  if FSectionWidth0 > 0 then
                   begin
                    MoveTo(FindSectionWidth(0),Header.Height);
                    LineTo(FindSectionWidth(0),Height);
                   end;
                  if FSectionWidth1 > 0 then
                   begin
                    MoveTo(FindSectionWidth(1),Header.Height);
                    LineTo(FindSectionWidth(1),Height);
                   end;
                  if FSectionWidth2 > 0 then
                   begin
                    MoveTo(FindSectionWidth(2),Header.Height);
                    LineTo(FindSectionWidth(2),Height);
                   end;
                  if FSectionWidth3 > 0 then
                   begin
                    MoveTo(FindSectionWidth(3),Header.Height);
                    LineTo(FindSectionWidth(3),Height);
                   end;
                  if FSectionWidth4 > 0 then
                   begin
                    MoveTo(FindSectionWidth(4),Header.Height);
                    LineTo(FindSectionWidth(4),Height);
                   end;
                  if FSectionWidth5 > 0 then
                   begin
                    MoveTo(FindSectionWidth(5),Header.Height);
                    LineTo(FindSectionWidth(5),Height);
                   end;
                  if FSectionWidth6 > 0 then
                   begin
                    MoveTo(FindSectionWidth(6),Header.Height);
                    LineTo(FindSectionWidth(6),Height);
                   end;
                  if FSectionWidth7 > 0 then
                   begin
                    MoveTo(FindSectionWidth(7),Header.Height);
                    LineTo(FindSectionWidth(7),Height);
                   end;
                  if FSectionWidth8 > 0 then
                   begin
                    MoveTo(FindSectionWidth(8),Header.Height);
                    LineTo(FindSectionWidth(8),Height);
                   end;
                  if FSectionWidth9 > 0 then
                   begin
                    MoveTo(FindSectionWidth(9),Header.Height);
                    LineTo(FindSectionWidth(9),Height);
                   end;
                  if FSectionWidth10 > 0 then
                   begin
                    MoveTo(FindSectionWidth(10),Header.Height);
                    LineTo(FindSectionWidth(10),Height);
                   end;
                  if FSectionWidth11 > 0 then
                   begin
                    MoveTo(FindSectionWidth(11),Header.Height);
                    LineTo(FindSectionWidth(11),Height);
                   end;
                  if FSectionWidth12 > 0 then
                   begin
                    MoveTo(FindSectionWidth(12),Header.Height);
                    LineTo(FindSectionWidth(12),Height);
                   end;
                  if FSectionWidth13 > 0 then
                   begin
                    MoveTo(FindSectionWidth(13),Header.Height);
                    LineTo(FindSectionWidth(13),Height);
                   end;
                  if FSectionWidth14 > 0 then
                   begin
                    MoveTo(FindSectionWidth(14),Header.Height);
                    LineTo(FindSectionWidth(14),Height);
                   end;
                 end;
        end;
       end
      else
       begin
        with Canvas do
        begin
         FillRect(Rect);
         TextOut(Rect.Left,Rect.Top+1,s);
        end;
       end;
end;

(*******************************************************
* procedure TMultiList.SetItemBitmap();
*
* Author: Jim Wood
*
* Purpose: This procedure is called when a bitmap is
*          loaded into a TBitmap type property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetItemBitmap(ListIndex: Integer; BitmapIndex: Integer);
var
 TempIndex: Integer;
 NewBitmap,
 NewString,
 s: string;
begin
   TempIndex := ItemIndex;
   try s := Items[ListIndex];
    begin
     case s[1] of
        '~',
        '0'..'9',
        '*',
        '!',
        '@',
        '#',
        '$',
        '%': NewString := Copy(s,2,Length(s)-1);
        else NewString := Copy(s,1,Length(s));
     end;
    end;
   except
   end;
   case BitmapIndex of
      0..9: NewBitmap := IntToStr(BitmapIndex);
      10: NewBitmap := '*';
      11: NewBitmap := '!';
      12: NewBitmap := '@';
      13: NewBitmap := '#';
      14: NewBitmap := '$';
      15: NewBitmap := '%';
      else NewBitmap := '';
   end;
    Items[ListIndex] := NewBitmap + NewString;
   ItemIndex := TempIndex;
end;

(*******************************************************
* procedure TMultiList.SetItemChild();
*
* Author: Jim Wood
*
* Purpose: This method is used to set a child item value.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetItemChild(ColIndex: Integer; ListIndex: Integer; ChangeItem: string);
var
 ExistingColumns,
 CountCols,
 ParseColumn,
 TempIndex: Integer;
 NewString,
 s: string;
begin
TempIndex := ItemIndex;
try s := Items[ListIndex];
 begin
     case s[1] of
        '~',
        '0'..'9',
        '*',
        '!',
        '@',
        '#',
        '$',
        '%':
           begin
            NewString := Copy(s,2,Length(s)-1);
            s := Copy(s,1,1);
           end
        else
           begin
            NewString := Copy(s,1,Length(s));
            s := '';
           end;
     end;
 end;
except
end;
ExistingColumns := 0;
for CountCols := 1 to Length(NewString) do
 begin
  if NewString[CountCols] = FSeperator then
   Inc(ExistingColumns);
 end;
while ExistingColumns <= ColIndex do
 begin
  NewString := NewString + FSeperator;
  Inc(ExistingColumns);
 end;
if ColIndex > 0 then
 begin
  for ParseColumn := 1 to ColIndex do
   begin
     s := s+Copy(NewString,1,Pos(FSeperator,NewString));
     NewString := Copy(NewString,Pos(FSeperator,NewString)+1,Length(NewString)-Pos(FSeperator,NewString)+1);
   end;
 end
else
 begin
     NewString := Copy(NewString,Pos(FSeperator,NewString)+1,Length(NewString)-Pos(FSeperator,NewString)+1);
 end;
Items[ListIndex] := s+ ChangeItem + FSeperator + Copy(NewString,Pos(FSeperator,NewString)+1,
                    Length(NewString)-Pos(FSeperator,NewString)+1);
ItemIndex := TempIndex;
end;

(*******************************************************
* procedure TMultiList.SetItemParent();
*
* Author: Jim Wood
*
* Purpose: This method is used to set the parent item value.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetItemParent(ListIndex: Integer; ChangeItem: string);
var
 TempIndex: Integer;
 NewString,
 s: string;
begin
TempIndex := ItemIndex;
try s := Items[ListIndex];
 begin
     case s[1] of
        '~',
        '0'..'9',
        '*',
        '!',
        '@',
        '#',
        '$',
        '%':
           begin
            NewString := Copy(s,2,Length(s)-1);
            s := Copy(s,1,1);
           end
        else
           begin
            NewString := Copy(s,1,Length(s));
            s := '';
           end;
     end;
 end;
except
end;
 NewString := Copy(NewString,Pos(FSeperator,NewString)+1,Length(NewString)-Pos(FSeperator,NewString)+1);
 Items[ListIndex] := s+ ChangeItem + FSeperator + NewString;
ItemIndex := TempIndex;
end;

(*******************************************************
* procedure TMultiList.ItemParent();
*
* Author: Jim Wood
*
* Purpose: This method is used to return the value of
*          the parent item at the index passed to it.
*
* Caveats: None
*
*******************************************************)
function TMultiList.ItemParent(ListIndex: Integer): string;
var
 s: string;
begin
try s := Items[ListIndex];
 begin
     if FIgnoreBitmap then
        begin
           Result := Copy(s,1,Pos(FSeperator,s)-1);
           Exit;
        end;
     case s[1] of
        '~',
        '0'..'9',
        '*',
        '!',
        '@',
        '#',
        '$',
        '%':
           begin
              Result := Copy(s,2,Pos(FSeperator,s)-2)
           end
        else
           begin
              Result := Copy(s,1,Pos(FSeperator,s)-1);
           end;
     end;
 end;
except
 Result := '';
end;
end;

(*******************************************************
* procedure TMultiList.ItemBitmap();
*
* Author: Jim Wood
*
* Purpose: This method is used to return the value of
*          the bitmap item at the index passed to it.
*
* Caveats: None
*
*******************************************************)
function TMultiList.ItemBitmap(ListIndex: Integer): Integer;
var
 s: string;
begin
try s := Items[ListIndex];
 begin
  case s[1] of
     '0'..'9': Result := StrToInt(Copy(s,1,1));
     '*': Result := 10;
     '!': Result := 11;
     '@': Result := 12;
     '#': Result := 13;
     '$': Result := 14;
     '%': Result := 15;
     else Result := -1;
  end;
 end;
except
 Result := -1;
end;
end;

(*******************************************************
* procedure TMultiList.ItemChild();
*
* Author: Jim Wood
*
* Purpose: This method is used to return the value of
*          the child item at the column, & index passed
*          to it.
*
* Caveats: None
*
*******************************************************)
function TMultiList.ItemChild(ColIndex: Integer; ListIndex: Integer): string;
var
 NewString,
 s: string;
 ParseColumn: Integer;
begin
try s := Items[ListIndex];
 begin
     case s[1] of
        '~',
        '0'..'9',
        '*',
        '!',
        '@',
        '#',
        '$',
        '%': NewString := Copy(s,2,Length(s)-1);
        else NewString := Copy(s,1,Length(s));
     end;
 end;
Result := '';
if ColIndex > 0 then
 begin
  for ParseColumn := 1 to ColIndex do
   begin
    s := Copy(NewString,Pos(FSeperator,NewString)+1,Length(NewString)-Pos(FSeperator,NewString)+1);
    NewString := s;
   end;
 end
else
 begin
  s := NewString;
 end;
Result := Copy(s,1,Pos(FSeperator,s)-1);
except
 Result := '';
end;
end;

(*******************************************************
* procedure TMultiList.SetColumnSeperator();
*
* Author: Jim Wood
*
* Purpose: This procedure is used to set the Column
*          Seperator.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetColumnSeperator(Value: string);
begin
 if FSeperator <> Value then
  FSeperator[1] := Value[1];
  Invalidate;
end;

(*******************************************************
* procedure TMultiList.DefineProperties();
*
* Author: Jim Wood
*
* Purpose: This procedure is used to define the Bitmap
*          array data.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineBinaryProperty('Data', ReadBinaryData, WriteBinaryData,
    FUserBitmaps <> []);
end;

(*******************************************************
* procedure TMultiList.ReadBinaryData();
*
* Author: Jim Wood
*
* Purpose: This procedure will read a Bitmap stream from
*          the data buffer.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.ReadBinaryData(Stream: TStream);
begin
  Stream.ReadBuffer(FOldBitmaps, SizeOf(FOldBitmaps));
end;

(*******************************************************
* procedure TMultiList.WriteBinaryData();
*
* Author: Jim Wood
*
* Purpose: This procedure will write a Bitmap stream to
*          the data buffer.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.WriteBinaryData(Stream: TStream);
begin
  Stream.WriteBuffer(FuserBitmaps, SizeOf(FUserBitmaps));
end;

(*******************************************************
* procedure TMultiList.CreateGlyph();
*
* Author: Jim Wood
*
* Purpose: This procedure will create the Bitmap Glyph.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.CreateGlyph;
var
  I: Integer;
begin
  FUserBitmaps := [];
  FOldBitmaps := [];
  for I := Low(FPictures) to High(FPictures) do
  begin
    FPictures[I] := TBitmap.Create;
    FPictures[I].Handle := LoadBitmap(HInstance, Images[I]);
  end;
end;

(*******************************************************
* procedure TMultiList.StoreBitmap();
*
* Author: Jim Wood
*
* Purpose: This procedure will store the Bitmap Glyph in
*          the Bitmap array.
*
* Caveats: None
*
*******************************************************)
function TMultiList.StoreBitmap(Index: Integer): Boolean;
begin
  Result := TListBitmap(Index) in FUserBitmaps;
end;

(*******************************************************
* procedure TMultiList.ClearBitmap();
*
* Author: Jim Wood
*
* Purpose: This procedure will clear a Bitmap Glyph from
*          the Bitmap array.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.ClearBitmap(var Bitmap: TBitmap; Kind: TListBitmap);
begin
  if Bitmap <> nil then
  begin
    Bitmap.Free;
    Bitmap := nil;
  end;
end;

(*******************************************************
* procedure TMultiList.ChangeBitmap();
*
* Author: Jim Wood
*
* Purpose: This procedure will change a Bitmap Glyph in
*          the Bitmap array.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.ChangeBitmap(Value: TBitmap; Kind: TListBitmap);
var
  Bitmap: ^TBitmap;
begin
  Bitmap := @FPictures[Ord(Kind)];
  Include(FUserBitmaps, Kind);
  if Value = nil then ClearBitmap(Bitmap^, Kind)
  else Bitmap^.Assign(Value);
  Invalidate;
end;

(*******************************************************
* procedure TMultiList.SetPicture();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the current Bitmap Glyph.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetPicture(Index: Integer; Value: TBitmap);
begin
  ChangeBitmap(Value, TListBitmap(Index));
end;

function TMultiList.GetAlignment(Index: Integer): TAlignment;
begin
  Result := FAlignment[Index];
end;

procedure TMultiList.SetAlignment(Index: Integer; Value: TAlignment);
begin
  if Value <> FAlignment[Index] then
  begin
    FAlignment[Index] := Value;
    Repaint;
  end;
end;

(*******************************************************
* procedure TMultiList.GetPicture();
*
* Author: Jim Wood
*
* Purpose: This procedure retrieves the Bitmap Glyph at
*          the given index and returns it to the caller.
*
* Caveats: None
*
*******************************************************)
function TMultiList.GetPicture(Index: Integer): TBitmap;
begin
  if csLoading in ComponentState then
    Include(FUserBitmaps, TListBitmap(Index));
  Result := FPictures[Index];
end;

(*******************************************************
* procedure TMultiList.SetHeaderSections();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSections of the
*          THeader := the local Section Width properties.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetHeaderSections(Value: TStringList);
begin
  if FHeaderSections <> Value then
   begin
     while Value.Count > MaxColumns do
      begin
       Value.Delete(Value.Count-1);
      end;
       FHeaderSections.Assign( Value );
       Header.Sections.Assign(FHeaderSections);
       FirstSize := TRUE;
       Invalidate;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth0(Value: Integer);
begin
  if FSectionWidth0 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth0 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth1(Value: Integer);
begin
  if FSectionWidth1 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth1 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth2(Value: Integer);
begin
  if FSectionWidth2 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth2 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth3(Value: Integer);
begin
  if FSectionWidth3 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth3 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth4(Value: Integer);
begin
  if FSectionWidth4 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth4 := Value;
   end;
end;

procedure TMultiList.SetIndexDefault(Value: Boolean);
begin
  if FIndexDefault <> Value then
   begin
     if Value then
        FIndexProperty := 1
     else
        FIndexProperty := 0;
     FIndexDefault := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth5(Value: Integer);
begin
  if FSectionWidth5 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth5 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth6(Value: Integer);
begin
  if FSectionWidth6 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth6 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth7(Value: Integer);
begin
  if FSectionWidth7 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth7 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth8(Value: Integer);
begin
  if FSectionWidth8 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth8 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth9(Value: Integer);
begin
  if FSectionWidth9 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth9 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth10(Value: Integer);
begin
  if FSectionWidth10 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth10 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth11(Value: Integer);
begin
  if FSectionWidth11 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth11 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth12(Value: Integer);
begin
  if FSectionWidth12 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth12 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth13(Value: Integer);
begin
  if FSectionWidth13 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth13 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetSectionWidthx();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderSection of the
*          THeader := the local Section Width property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetSectionWidth14(Value: Integer);
begin
  if FSectionWidth14 <> Value then
   begin
     FirstSize := TRUE;
     FSectionWidth14 := Value;
   end;
end;

(*******************************************************
* procedure TMultiList.SetHeaderHeight();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the HeaderHeight of the
*          THeader := the local header height property.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetHeaderHeight(Value: Integer);
begin
  if FHeaderHeight <> Value then
   begin
     FHeaderHeight := Value;
     Header.Height := FHeaderHeight;
     Invalidate;
   end;
end;

(*******************************************************
* procedure TMultiList.SetScrollSpeed();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the scroll speed in pixels
*          of the header resizing event (1-5).
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetScrollSpeed(Value: Integer);
begin
  if FScrollSpeed <> Value then
   begin
     FScrollSpeed := Value;
     if Value > 5 then FScrollSpeed := 5;
     if Value < 1 then FScrollSpeed := 1;
   end;
end;

(*******************************************************
* procedure TMultiList.SetShowParentHighlight();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the FShowParentHighlight
*          flag.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetShowParentHighlight(Value: Boolean);
begin
  if FShowParentHighlight <> Value then
   begin
     FShowParentHighlight := Value;
     Invalidate;
   end;
end;

(*******************************************************
* procedure TMultiList.SetShowLines();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the FShowColLines
*          flag and redraws the listbox.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetShowLines(Value: Boolean);
begin
  if FShowColLines <> Value then
   begin
     FShowColLines := Value;
     Invalidate;
   end;
end;

(*******************************************************
* procedure TMultiList.SetHScroll();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the FHScroll flag for a
*          horizontal scroll bar and redraws the listbox.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetHScroll(Value: Boolean);
begin
  if FHScroll <> Value then
   begin
     FHScroll := Value;
     Invalidate;
   end;
end;

(*******************************************************
* procedure TMultiList.SetScrollWidth();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the FScrollWidth flag for the
*          horizontal scroll width and redraws the listbox.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetScrollWidth(Value: Integer);
begin
  if FScrollWidth <> Value then
   begin
     FScrollWidth := Value;
     Invalidate;
   end;
end;

(*******************************************************
* procedure TMultiList.SetHeaderFont();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the font of the THeader
*          created in the listbox.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetHeaderFont(Value: TFont);
begin
  if FHeaderFont <> Value then
   begin
     FHeaderFont.Assign( Value );
     Header.Font.Assign(FHeaderFont);
     Invalidate;
   end;
end;

(*******************************************************
* procedure TMultiList.SetColLineColor();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the seperator line color.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetColLineColor(Value: TColor);
begin
  if FColLineColor <> Value then
   begin
     FColLineColor := Value;
     Invalidate;
   end;
end;

(*******************************************************
* procedure TMultiList.SetHighlightColor();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the selected highlight color.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetHighlightColor(Value: TColor);
begin
  if FHighlightColor <> Value then
   begin
     FHighlightColor := Value;
     Invalidate;
   end;
end;

(*******************************************************
* procedure TMultiList.SetChildHighlightTextColor();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the selected child text
*          highlight color.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetChildHighlightTextColor(Value: TColor);
begin
  if FChildHighlightTextColor <> Value then
   begin
     FChildHighlightTextColor := Value;
     Invalidate;
   end;
end;

(*******************************************************
* procedure TMultiList.SetHighlightText();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the default text
*          highlight color.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetHighlightText(Value: TColor);
begin
  if FHighlightText <> Value then
   begin
     FHighlightText := Value;
     Invalidate;
   end;
end;

(*******************************************************
* procedure TMultiList.SetMaskColor();
*
* Author: Jim Wood
*
* Purpose: This procedure sets the color to use as a
*          mask on all bitmap items.
*
* Caveats: None
*
*******************************************************)
procedure TMultiList.SetMaskColor(Value: TColor);
begin
  if FMaskColor <> Value then
   begin
     FMaskColor := Value;
     Invalidate;
   end;
end;

(*******************************************************
* destructor TMultiList.Destroy();
*
* Author: Jim Wood
*
* Purpose: This procedure cleans up after itself.
*
* Caveats: None
*
*******************************************************)
destructor TMultiList.Destroy;
var
  I: Integer;
begin
  if Header <> nil then Header.Free;
  FHeaderFont.Free;
  FHeaderSections.Free;
  FAbout.Free;
  for I := Low(FPictures) to High(FPictures) do FPictures[I].Free;
  inherited Destroy;
end;
end.

